diff --git a/pgadmin/ctl/ctlSQLResult.cpp b/pgadmin/ctl/ctlSQLResult.cpp
index 2b3759c..e701ea0 100644
--- a/pgadmin/ctl/ctlSQLResult.cpp
+++ b/pgadmin/ctl/ctlSQLResult.cpp
@@ -27,7 +27,7 @@ ctlSQLResult::ctlSQLResult(wxWindow *parent, pgConn *_conn, wxWindowID id, const
 	: ctlSQLGrid(parent, id, pos, size)
 {
 	conn = _conn;
-	thread = 0;
+	thread = NULL;
 
 	SetTable(new sqlResultTable(), true);
 
@@ -46,10 +46,10 @@ ctlSQLResult::~ctlSQLResult()
 	if (thread)
 	{
 		thread->CancelExecution();
-		thread->Wait();
+		if (thread->IsRunning())
+			thread->Wait();
 
 		delete thread;
-
 		thread = NULL;
 	}
 }
@@ -133,13 +133,15 @@ int ctlSQLResult::Execute(const wxString &query, int resultToRetrieve, wxWindow
 	if (thread)
 	{
 		thread->CancelExecution();
-		thread->Wait();
+		if (thread->IsRunning())
+		{
+			thread->Wait();
+		}
 
 		delete thread;
 		thread = NULL;
 	}
 
-
 	thread = new pgQueryThread(conn, query, resultToRetrieve, caller, eventId, data);
 
 	if (thread->Create() != wxTHREAD_NO_ERROR)
@@ -161,8 +163,13 @@ int ctlSQLResult::Abort()
 	{
 		((sqlResultTable *)GetTable())->SetThread(0);
 
-		thread->CancelExecution();
-		thread->Wait();
+		if (thread->IsRunning())
+		{
+			thread->CancelExecution();
+			thread->Wait();
+		}
+
+		delete thread;
 	}
 	return 0;
 }
@@ -269,7 +276,10 @@ wxString ctlSQLResult::GetErrorMessage()
 
 pgError ctlSQLResult::GetResultError()
 {
-	return thread->GetResultError();
+	if (thread)
+		return thread->GetResultError();
+	pgError dummy;
+	return dummy;
 }
 
 long ctlSQLResult::NumRows() const
@@ -392,7 +402,7 @@ wxString sqlResultTable::GetValue(int row, int col)
 
 sqlResultTable::sqlResultTable()
 {
-	thread = 0;
+	thread = NULL;
 }
 
 int sqlResultTable::GetNumberRows()
diff --git a/pgadmin/debugger/dbgEvents.cpp b/pgadmin/debugger/dbgEvents.cpp
index bac466f..ae04b5f 100644
--- a/pgadmin/debugger/dbgEvents.cpp
+++ b/pgadmin/debugger/dbgEvents.cpp
@@ -206,8 +206,8 @@ void dbgController::ResultTargetComplete(pgQueryResultEvent &_ev)
 		if (m_execConnThread->IsRunning())
 		{
 			m_execConnThread->CancelExecution();
+			m_execConnThread->Wait();
 		}
-		m_execConnThread->Wait();
 
 		pgConn *execConn = m_execConnThread->GetConn();
 		delete m_execConnThread;
diff --git a/pgadmin/debugger/dlgDirectDbg.cpp b/pgadmin/debugger/dlgDirectDbg.cpp
index cc0cba9..464f3a2 100644
--- a/pgadmin/debugger/dlgDirectDbg.cpp
+++ b/pgadmin/debugger/dlgDirectDbg.cpp
@@ -520,7 +520,8 @@ void dlgDirectDbg::OnCancel(wxCommandEvent &_ev)
 	{
 		m_thread->CancelEval();
 
-		m_thread->Wait();
+		if (m_thread->IsRunning())
+			m_thread->Wait();
 
 		delete m_thread;
 		m_thread = NULL;
@@ -757,7 +758,8 @@ void dlgDirectDbg::ResultArgsUpdated(wxCommandEvent &_ev)
 	if (m_thread)
 	{
 		m_thread->CancelEval();
-		m_thread->Wait();
+		if (m_thread->IsRunning())
+			m_thread->Wait();
 
 		delete m_thread;
 		m_thread = NULL;
@@ -781,8 +783,8 @@ void dlgDirectDbg::ResultArgsUpdateError(wxCommandEvent &_ev)
 			if (m_thread)
 			{
 				m_thread->CancelEval();
-
-				m_thread->Wait();
+				if (m_thread->IsRunning())
+					m_thread->Wait();
 
 				delete m_thread;
 				m_thread = NULL;
@@ -799,7 +801,8 @@ void dlgDirectDbg::ResultArgsUpdateError(wxCommandEvent &_ev)
 	if (m_thread)
 	{
 		m_thread->CancelEval();
-		m_thread->Wait();
+		if (m_thread->IsRunning())
+			m_thread->Wait();
 
 		delete m_thread;
 		m_thread = NULL;
